Cursor

Struct Cursor 

pub struct Cursor<'c, T> { /* private fields */ }
Cursor.

This struct is created by the cursor method on CircularList.

Implementations

Source

impl<'c, T> Cursor<'c, T>

pub fn index(&self) -> usize

Returns the cursor position index within the CircularList.

pub fn move_next(&mut self)

Moves the cursor to the next element of the CircularList.

If the cursor is pointing to the last element then this will move it to the first element of the CircularList.

pub fn move_prev(&mut self)

Moves the cursor to the previous element of the CircularList.

If the cursor is pointing to the first element then this will move it to the last element of the CircularList.

pub fn current(&self) -> &'c T

Returns a reference to the element that the cursor is currently pointing to.

pub fn peek_next(&self) -> &'c T

Returns a reference to the next element.

pub fn peek_prev(&self) -> &'c T

Returns a reference to the previous element.