pub struct Cursor<'c, T> { /* private fields */ }Expand description
Cursor.
This struct is created by the cursor method on CircularList.
Implementations
Sourceimpl<'c, T> Cursor<'c, T>
impl<'c, T> Cursor<'c, T>
Sourcepub fn move_next(&mut self)
pub fn move_next(&mut self)
Moves the cursor to the next element of the CircularList.
If the cursor is pointing to the last element then this will move it to
the first element of the CircularList.
Sourcepub fn move_prev(&mut self)
pub fn move_prev(&mut self)
Moves the cursor to the previous element of the CircularList.
If the cursor is pointing to the first element then this will move it to
the last element of the CircularList.